﻿using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;

using TestLibrary.Business;

namespace TestLibrary.ViewModels
{
    /// <summary>
    /// 사람 뷰 모델
    /// </summary>
    public class PersonViewModel : BindableBase, INavigationAware
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 영역 탐색 저널 인터페이스
        /// </summary>
        private IRegionNavigationJournal journal;

        /// <summary>
        /// 선택한 사람
        /// </summary>
        private Person selectedPerson;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택한 사람 - SelectedPerson

        /// <summary>
        /// 선택한 사람
        /// </summary>
        public Person SelectedPerson
        {
            get
            {
                return this.selectedPerson;
            }
            set
            {
                SetProperty(ref this.selectedPerson, value);
            }
        }

        #endregion
        #region 뒤로 돌아가기 명령 - GoBackCommand

        /// <summary>
        /// 뒤로 돌아가기 명령
        /// </summary>
        public DelegateCommand GoBackCommand { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PersonViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public PersonViewModel()
        {
            GoBackCommand = new DelegateCommand(GoBack);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 탐색 타겟 여부 구하기 - IsNavigationTarget(navigationContext)

        /// <summary>
        /// 탐색 타겟 여부 구하기
        /// </summary>
        /// <param name="navigationContext">탐색 컨텍스트</param>
        /// <returns>탐색 타겟 여부</returns>
        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            Person person = navigationContext.Parameters["person"] as Person;

            if(person != null)
            {
                return SelectedPerson != null && SelectedPerson.LastName == person.LastName;
            }
            else
            {
                return true;
            }
        }

        #endregion
        #region 탐색 진입시 처리하기 - OnNavigatedTo(navigationContext)

        /// <summary>
        /// 탐색 진입시 처리하기
        /// </summary>
        /// <param name="navigationContext">탐색 컨텍스트</param>
        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            this.journal = navigationContext.NavigationService.Journal;

            Person person = navigationContext.Parameters["person"] as Person;

            if(person != null)
            {
                SelectedPerson = person;
            }
        }

        #endregion
        #region 탐색 이탈시 처리하기 - OnNavigatedFrom(navigationContext)

        /// <summary>
        /// 탐색 이탈시 처리하기
        /// </summary>
        /// <param name="navigationContext">탐색 컨텍스트</param>
        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 뒤로 돌아가기 - GoBack()

        /// <summary>
        /// 뒤로 돌아가기
        /// </summary>
        private void GoBack()
        {
            this.journal.GoBack();
        }

        #endregion
    }
}